package xamora.gp_dbc.commands;

import JinRyuu.JRMCore.server.JGPlayerMP;
import com.forgeessentials.api.UserIdent;
import net.minecraft.command.ICommand;
import net.minecraft.command.ICommandSender;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.server.MinecraftServer;
import net.minecraft.util.ChatComponentText;
import net.minecraft.util.EnumChatFormatting;

import java.util.ArrayList;
import java.util.List;

import static com.forgeessentials.core.commands.ForgeEssentialsCommandBase.getListOfStringsMatchingLastWord;
import static xamora.gp_dbc.Main.gp;

public class CommandGP implements ICommand {

    private final List aliases;
    public static final String[] COMMANDS = {
            "addGP",
            "removeGP",
            "inmate",
            "freeInmate",
            "removeJail",
            "addCriminal",
            "removeCriminal",
            "banGP",
            "removeBanGP",
    };

    public CommandGP() {
        aliases = new ArrayList();
        aliases.add("galacticpatrol");
        aliases.add("gp");

    }

    @Override
    public String getCommandName() {
        return "galacticpatrol";
    }

    @Override
    public String getCommandUsage(ICommandSender p_71518_1_) {
        return "gp <text> [<player>]";
    }

    @Override
    public List getCommandAliases() {
        return this.aliases;
    }

    @Override
    public void processCommand(ICommandSender sender, String[] argString) {

        EntityPlayer player = null;
        String arg;
        List<String> args = new ArrayList<>();

        if (argString.length == 0) {
            sender.addChatMessage(new ChatComponentText(EnumChatFormatting.RED + "/gp <text> [<player>]"));
            return;
        }
        else if (argString.length == 1) {
            if (sender instanceof EntityPlayer)
                player = (EntityPlayer) sender;
            else {
                sender.addChatMessage(new ChatComponentText(EnumChatFormatting.RED + "/gp <text> [<player>]"));
                return;
            }
        }
        else {
            player = UserIdent.getPlayerByMatchOrUsername(sender, argString[argString.length - 1]);

            if (player == null) {
                for (int i = 1; i < argString.length; i++)
                    args.add(argString[i]);

                if (sender instanceof EntityPlayer)
                    player = (EntityPlayer) sender;
                else {
                    sender.addChatMessage(new ChatComponentText(EnumChatFormatting.RED + "Player not found"));
                    return;
                }
            }
            else
                for (int i = 1; i < argString.length - 1; i++)
                    args.add(argString[i]);
        }
        arg = argString[0];

        JGPlayerMP jgPlayer = new JGPlayerMP(player);
        jgPlayer.connectBaseNBT();

        int error = 0;

        if (arg.equals(COMMANDS[0])) // Add
            error = gp.addToGalaticPatrol(player, true);
        else if (arg.equals(COMMANDS[1])) // Remove
            error = gp.removeFromGalacticPatrol(player);
        else if (arg.equals(COMMANDS[2])) { // Inmate
            if (args.size() != 1) {
                sender.addChatMessage(new ChatComponentText(EnumChatFormatting.RED + "/gp inmate sec [<player>]"));
                return;
            }
            else {
                try {
                    error = gp.jail.putPlayerInJail(sender, player, Long.parseLong(args.get(0)), true) != null ? 0 : 1;
                } catch (Exception e) {
                    sender.addChatMessage(new ChatComponentText(EnumChatFormatting.RED + "/gp inmate sec [<player>]"));
                    return;
                }
            }
        }
        else if (arg.equals(COMMANDS[3])) { // freeInmate
            error = gp.jail.removePlayerFromJail(sender, player, true);
        }
        else if (arg.equals(COMMANDS[4])) {// RemoveJail
            if (args.size() == 1) {
                try {
                    error = gp.jail.removeCell(Integer.parseInt(args.get(0)));
                } catch (Exception e) {
                    sender.addChatMessage(new ChatComponentText(EnumChatFormatting.RED + "/gp removeJail <id>"));
                }
            }
            else if (args.size() == 4) {
                try {
                    error = gp.jail.removeCell(Integer.parseInt(args.get(0)), Integer.parseInt(args.get(1)), Integer.parseInt(args.get(2)), Integer.parseInt(args.get(3)));
                } catch (Exception e) {
                    sender.addChatMessage(new ChatComponentText(EnumChatFormatting.RED + "/gp removeJail <x> <y> <z> <dimension>"));
                }
            }
            else {
                sender.addChatMessage(new ChatComponentText(EnumChatFormatting.RED + "/gp removeJail <id>"));
                sender.addChatMessage(new ChatComponentText(EnumChatFormatting.RED + "/gp removeJail <x> <y> <z> <dimension>"));
                return;
            }
        }
        else if (arg.equals(COMMANDS[5])) { // addCriminal
            long rate = 0;

            if (args.size() == 1) {
                try {
                    rate = Long.parseLong(args.get(0));
                } catch (Exception e) {
                    System.out.println(e);
                }
            }
            error = gp.criminals.addCriminal(player, rate) != null ? 0 : 1;
        }
        else if (arg.equals(COMMANDS[6])) { // removeCriminal
            error = gp.criminals.removeCriminal(player.getUniqueID()) ? 0 : 1;
        }
        else if (arg.equals(COMMANDS[7])) { // banGP
            try {
                error = gp.banFromGalacticPatrol(player, "", Long.parseLong(args.get(0)));
            } catch (Exception e) {
                sender.addChatMessage(new ChatComponentText(EnumChatFormatting.RED + "/gp banGP <sec> [<player>]"));
                error = 1;
            }

        }
        else if (arg.equals(COMMANDS[8])) { // removeBanGP
            error = gp.bannedFromGP.remove(player.getUniqueID()) == null ? 1 : 0;
        }
        else {
            sender.addChatMessage(new ChatComponentText(EnumChatFormatting.RED.toString() + EnumChatFormatting.UNDERLINE.toString() + "Galactic Patrol:"
                    + EnumChatFormatting.RESET.toString() + EnumChatFormatting.RED.toString() + " Command not found"));
            return;
        }

        if (error == 0)
            sender.addChatMessage(new ChatComponentText(EnumChatFormatting.GREEN + "Command success"));
        else
            sender.addChatMessage(new ChatComponentText(EnumChatFormatting.RED + "Command failed"));

    }

    @Override
    public boolean canCommandSenderUseCommand(ICommandSender sender) {
        if (sender instanceof EntityPlayer)
            return sender.canCommandSenderUseCommand(4, "");
        return true;
    }

    @Override
    public List<?> addTabCompletionOptions(ICommandSender sender, String[] strArray) {
        List<String> strListComp = new ArrayList<String>();

        List<EntityPlayer> listPlayer = MinecraftServer.getServer().getConfigurationManager().playerEntityList;

        if (strArray.length > 1)
            for (EntityPlayer player : listPlayer)
                strListComp.add(player.getDisplayName());
        else if (strArray.length == 1)
            return getListOfStringsMatchingLastWord(strArray, COMMANDS);

        return strListComp;
    }

    @Override
    public boolean isUsernameIndex(String[] p_82358_1_, int p_82358_2_) {
        return false;
    }

    @Override
    public int compareTo(Object o) {
        return 0;
    }

}
