package xamora.gp_dbc.system;

import JinRyuu.JRMCore.server.JGPlayerMP;
import cpw.mods.fml.common.eventhandler.SubscribeEvent;
import cpw.mods.fml.common.gameevent.PlayerEvent;
import cpw.mods.fml.common.gameevent.TickEvent;
import cpw.mods.fml.relauncher.Side;
import cpw.mods.fml.relauncher.SideOnly;
import net.minecraft.entity.Entity;
import net.minecraftforge.event.entity.living.LivingDeathEvent;
import xamora.gp_dbc.config.ConfigManager;
import xamora.gp_dbc.utils.Data;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.entity.player.EntityPlayerMP;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.server.MinecraftServer;
import net.minecraft.util.ChatComponentText;
import net.minecraft.util.EnumChatFormatting;
import net.minecraftforge.event.entity.living.LivingHurtEvent;
import xamora.gp_dbc.utils.TimeUtils;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.UUID;

import static net.minecraft.util.StatCollector.translateToLocal;
import static xamora.gp_dbc.Main.gp;
import static xamora.gp_dbc.config.ConfigManager.*;

public class GalacticPatrol {

    private List<UUID> patrollers;
    public Jail jail;
    public Criminals criminals;

    public HashMap<UUID, Long> InFightWithPatrollers = new HashMap<>();
    public HashMap<UUID, Long> bannedFromGP = new HashMap<>();

    public GalacticPatrol() {
        patrollers = Data.loadDataUUID("galactic_patrols");
        jail = new Jail();
        criminals = new Criminals();
        try {
            listStringToBanned(Data.loadDataString("banned"));
        } catch (Exception e) {
            System.out.println(e);
        };
    }

    public int addToGalaticPatrol(EntityPlayer player) {
        return addToGalaticPatrol(player, false);
    }

    public int addToGalaticPatrol(EntityPlayer player, boolean chatError) {
        return addToGalaticPatrol((EntityPlayerMP)player, chatError);
    }

    public int addToGalaticPatrol(EntityPlayerMP player) {
        return addToGalaticPatrol(player, false);
    }

    public int addToGalaticPatrol(EntityPlayerMP player, boolean chatError) {

        JGPlayerMP jgPlayer = new JGPlayerMP(player);
        jgPlayer.connectBaseNBT();
        int align = jgPlayer.getAlignment();


        if (align < ConfigManager.MIN_ALIGN_TO_JOIN) {
            if (chatError)
                player.addChatMessage(new ChatComponentText(EnumChatFormatting.RED + translateToLocal("error.min_align_to_join")));
            return 2;
        }
        else if (align > ConfigManager.MAX_ALIGN_TO_JOIN) {
            if (chatError)
                player.addChatMessage(new ChatComponentText(EnumChatFormatting.RED + translateToLocal("error.max_align_to_join")));
            return 3;
        }

        Criminal criminal = criminals.getCriminal(player.getUniqueID());
        if (criminal != null) {
            if (criminal.getRate() > 0) {
                if (chatError)
                    player.addChatMessage(new ChatComponentText(EnumChatFormatting.RED + translateToLocal("error.criminal_try_join_galactic_patrol")));
                return 4;
            }
        }

        if (bannedFromGP.containsKey(player.getUniqueID())) {
            long time_player = bannedFromGP.get(player.getUniqueID());
            long time = System.currentTimeMillis();

            if (bannedFromGP.get(player.getUniqueID()) > System.currentTimeMillis()) {

                if (chatError) {
                    long second_left = (bannedFromGP.get(player.getUniqueID()) - System.currentTimeMillis()) / 1000L;
                    player.addChatMessage(new ChatComponentText(EnumChatFormatting.RED + translateToLocal("error.banned_from_galactic_patrol")));
                    player.addChatMessage(new ChatComponentText(EnumChatFormatting.RED + TimeUtils.getTimeHours(second_left)));
                }
                return 5;
            }
            else
                bannedFromGP.remove(player.getUniqueID());
        }

        if (!patrollers.contains(player.getUniqueID())) {
            patrollers.add(player.getUniqueID());
            player.addChatMessage(new ChatComponentText(EnumChatFormatting.GREEN + translateToLocal("success.join_galactic_patrol")));
        }
        else {
            player.addChatMessage(new ChatComponentText(EnumChatFormatting.RED + translateToLocal("error.join_galactic_patrol")));
            return 6;
        }

        Data.saveDataUUID(patrollers, "galactic_patrols");

        return 0;
    }

    private void addForceToGalacticPatrol(UUID uuid) {
        patrollers.add(uuid);
        Data.saveDataUUID(patrollers, "galactic_patrols");
    }

    public int removeFromGalacticPatrol(EntityPlayer player) {
        return removeFromGalacticPatrol((EntityPlayerMP)player);
    }

    public int removeFromGalacticPatrol(EntityPlayerMP player) {

        boolean removed = patrollers.remove(player.getUniqueID());
        Data.saveDataUUID(patrollers, "galactic_patrols");

        if (removed)
            player.addChatMessage(new ChatComponentText(EnumChatFormatting.GREEN + translateToLocal("success.leave_galactic_patrol")));
        else
            player.addChatMessage(new ChatComponentText(EnumChatFormatting.RED + translateToLocal("error.leave_galactic_patrol")));

        return removed ? 0 : 1;
    }

    // time in second
    public int banFromGalacticPatrol(EntityPlayer banned, String reason, long time) {
        if (removeFromGalacticPatrol(banned) == 1)
            return 1;
        bannedFromGP.put(banned.getUniqueID(), System.currentTimeMillis() + time * 1000L);

        banned.addChatMessage(new ChatComponentText(EnumChatFormatting.RED + translateToLocal("error.banned_from_galactic_patrol") + " " + TimeUtils.getTimeHours(time)));
        if (reason.length() > 0)
            banned.addChatMessage(new ChatComponentText(EnumChatFormatting.RED + translateToLocal(reason)));

        Data.saveDataString(listBannedToListString(), "banned");
        return 0;
    }

    private void listStringToBanned(List<String> banned_str) throws Exception {
        for (String s : banned_str) {

            String[] split = s.split("~");
            if (split.length != 7)
                throw new Exception("Error loading " + ConfigManager.DIR_SAVE + "Inmates.txt");

            try {
                bannedFromGP.put(UUID.fromString(split[0]), Long.parseLong(split[1]));
            } catch (Exception e) {
                System.out.println(e);
                throw e;
            }
        }
    }

    private List<String> listBannedToListString() {

        List<String> banned_str = new ArrayList<>();
        for (UUID uuid : bannedFromGP.keySet()) {
            long time = bannedFromGP.get(uuid);

            banned_str.add(uuid + "~" + time);
        }

        return banned_str;
    }


    public boolean isInGalacticPatrol(EntityPlayer player) {
        return patrollers.contains(player.getUniqueID());
    }

    public boolean isInGalacticPatrol(EntityPlayerMP player) {
        return patrollers.contains(player.getUniqueID());
    }

    public boolean isInGalacticPatrol(UUID uuid) {
        return patrollers.contains(uuid);
    }

    public List<UUID> getPatrollers() { return patrollers; }

    @SubscribeEvent
    @SideOnly(Side.SERVER)
    public void checkReincarnated(TickEvent.ServerTickEvent event) {
        List<EntityPlayer> playerList = MinecraftServer.getServer().getConfigurationManager().playerEntityList;

        for (EntityPlayer player : playerList) {
            JGPlayerMP jgPlayer = new JGPlayerMP(player);
            NBTTagCompound nbt = jgPlayer.connectBaseNBT();
            boolean isReincarnated = nbt.getInteger("jrmcRencrnt") == 1;

            if (isReincarnated) {
                if (ConfigManager.REINCARNATION_RESET_GP && isInGalacticPatrol(player))
                    removeFromGalacticPatrol(player);
                if (ConfigManager.REINCARNATION_RESET_CRIMINAL && criminals.isCriminal(player.getUniqueID()))
                    criminals.removeCriminal(player.getUniqueID());
            }
        }
    }

    @SubscribeEvent
    @SideOnly(Side.SERVER)
    public void checkFight(LivingHurtEvent event) {
        if (!ConfigManager.CRIMINAL_QUIT_IN_FIGHT_GO_TO_JAIL)
            return;

        if (!(event.source.getSourceOfDamage() instanceof EntityPlayer) || !(event.entity instanceof EntityPlayer))
            return;

        EntityPlayer source = (EntityPlayer) event.source.getSourceOfDamage();
        EntityPlayer target = (EntityPlayer) event.entity;

        if (isInGalacticPatrol(source) && isInGalacticPatrol(target))
            return;
        if (!isInGalacticPatrol(source) && !isInGalacticPatrol(target))
            return;

        if (criminals.isCriminal(source.getUniqueID()))
            InFightWithPatrollers.put(source.getUniqueID(), System.currentTimeMillis());
        else if (criminals.isCriminal(target.getUniqueID()))
            InFightWithPatrollers.put(target.getUniqueID(), System.currentTimeMillis());
    }

    @SubscribeEvent
    @SideOnly(Side.SERVER)
    public void checkFightLogOut(PlayerEvent.PlayerLoggedOutEvent event) {
        if (!ConfigManager.CRIMINAL_QUIT_IN_FIGHT_GO_TO_JAIL)
            return;

        long currentTime = System.currentTimeMillis();
        Long time = InFightWithPatrollers.get(event.player.getUniqueID());
        if (time == null || currentTime - time > ConfigManager.CRIMINAL_TIME_TO_QUIT_IN_FIGHT)
            return;

        EntityPlayer jailer = null;
        if (event.player.getLastAttacker() instanceof EntityPlayer) {
            EntityPlayer attacker = (EntityPlayer) event.player.getLastAttacker();
            if (isInGalacticPatrol(attacker))
                jailer = attacker;
        }

        jail.putPlayerInJail(jailer, event.player);
    }

    // Ban
    @SubscribeEvent
    @SideOnly(Side.SERVER)
    public void checkAlignment(TickEvent.ServerTickEvent event) {
        List<EntityPlayer> playerList = MinecraftServer.getServer().getConfigurationManager().playerEntityList;

        for (EntityPlayer player : playerList) {
            if (!isInGalacticPatrol(player))
                return;

            JGPlayerMP jgPlayer = new JGPlayerMP(player);
            NBTTagCompound nbt = jgPlayer.connectBaseNBT();
            int align = jgPlayer.getAlignment();

            if (align < MIN_ALIGN_TO_JOIN)
                banFromGalacticPatrol(player, "error.banned_for_being_bad", BAN_TIME_BEING_BAD);
        }
    }

}
